import re
from itertools import chain, combinations
from PySimpleAutomata import automata_IO

# https://docs.python.org/3/library/itertools.html#recipes
def powerset(iterable):
    "powerset([1,2,3]) --> () (1,) (2,) (3,) (1,2) (1,3) (2,3) (1,2,3)"
    s = list(set(iterable))
    combs = chain.from_iterable(combinations(s, r) for r in range(len(s)+1))
    res = set(frozenset(x) for x in combs)
    # res = map(frozenset, combs)
    return res


def my_str(obj):
    s = str(obj)
    s = re.sub("frozenset\(\)", "{}", s)
    s = re.sub("frozenset\(\{(.*)\}\)", "{\g<1>}", s)
    return s

def _to_PySimpleAutomata_nfa(nfa:dict):
    """Generate a nfa (according to PySimpleAutomata specifications)
    given a NFA dictionary generated by
    the method LDLf_EmptyTraces.to_nfa"""
    str = my_str
    res = {}
    res["alphabet"] = set(map(str, nfa["alphabet"]))
    res["states"] = set(map(str, nfa["states"]))
    res["initial_states"] = set(map(str, nfa["initial_states"]))
    res["accepting_states"] = set(map(str, nfa["accepting_states"]))

    # more destinations for the same start state and action
    import collections
    transitions = collections.defaultdict(list)
    for v in nfa["transitions"]:
        transitions[(str(v[0]), str(v[1]))].append(str(v[2]))

    res["transitions"] = transitions
    return res


def _to_PySimpleAutomata_dfa(nfa:dict):
    """Generate a DFA (according to PySimpleAutomata specifications)
        given a NFA dictionary generated by
        the method LDLf_EmptyTraces.to_nfa"""
    res = _to_PySimpleAutomata_nfa(nfa)
    res = _to_PySimpleAutomata_dfa(res)
    return res


def print_nfa(nfa:dict, name, path="./"):
    """Converts the NFA into a PySimpleAutomata.NFA
    Then draw the NFA in .svg format.
    :param nfa: 
    """
    res = _to_PySimpleAutomata_nfa(nfa)
    automata_IO.nfa_to_dot(res, name, path)

def print_dfa(nfa:dict, name, path="./"):
    """Converts the NFA into a PySimpleAutomata.DFA
    Then draw the NFA in .svg format."""
    res = _to_PySimpleAutomata_dfa(nfa)
    automata_IO.nfa_to_dot(res, name, path)
